@@ -6,7 +6,21 @@
 #ifndef DEBUG_H
 #define DEBUG_H
 
-#undef DEBUG
+#include "ocdebug.h"
+
+#ifdef DAPDEBUG
+#  define DEBUG
+#  if DAPDEBUG > 0
+#    define DEBUG1
+#  endif
+#  if DAPDEBUG > 1
+#    define DEBUG2
+#  endif
+#  if DAPDEBUG > 2
+#    define DEBUG3
+#  endif
+#endif
+
 #undef PARSEDEBUG
 
 #include <stdarg.h>
@@ -43,5 +57,17 @@ extern int dapthrow(int err);
 #define THROWCHK(e)
 #endif
 
+#ifdef DEBUG
+#define SHOWFETCH (1)
+#define LOG0(level,msg) fprintf(stderr,msg)
+#define LOG1(level,msg,a1) fprintf(stderr,msg,a1)
+#define LOG2(level,msg,a1,a2) fprintf(stderr,msg,a1,a2)
+#else
+#define SHOWFETCH FLAGSET(nccomm->controls,NCF_SHOWFETCH)
+#define LOG0(level,msg) nclog(level,msg)
+#define LOG1(level,msg,a1) nclog(level,msg,a1)
+#define LOG2(level,msg,a1,a2) nclog(level,msg,a1,a2)
+#endif
+
 #endif /*DEBUG_H*/
 
