@@ -4,9 +4,7 @@
  *   $Header: /upc/share/CVS/netcdf-3/libncdap3/constraints3.c,v 1.40 2010/05/27 21:34:07 dmh Exp $
  *********************************************************************/
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -24,16 +22,10 @@
 
 #define DEBUG
 
-#ifndef nulldup
-#define nulldup(s) ((s)==NULL?NULL:strdup(s))
-#endif
-#ifndef nullfree
-#define nullfree(s) {if((s)!=NULL) {free(s);} else {}}
-#endif
+int dceverbose = 0;
 
 static char* opstrings[] = OPSTRINGS ;
 
-static int mergeprojection(DCEprojection* dst, DCEprojection* src);
 static void ceallnodesr(DCEnode* node, NClist* allnodes, CEsort which);
 
 /* Parse incoming url constraints, if any,
@@ -137,12 +129,18 @@ dceslicemerge(DCEslice* dst, DCEslice* src)
     int err = NC_NOERR;
     DCEslice tmp;
 
-    tmp.stride   = (dst->stride * src->stride);
-    tmp.first    = (dst->first+((src->first)*(dst->stride)));
-    tmp.length   = (((src->length - 1) / src->stride) * tmp.stride) + 1;
-    tmp.stop     = tmp.first + tmp.length;
-    tmp.count    = tmp.length / tmp.stride;
-    tmp.declsize = dst->declsize;
+    tmp.node.sort = CES_SLICE;
+    tmp.stride    = (dst->stride * src->stride);
+    tmp.first     = (dst->first+((src->first)*(dst->stride)));
+    tmp.length    = (((src->length - 1) / src->stride) * tmp.stride) + 1;
+    tmp.stop      = tmp.first + tmp.length;
+    tmp.count     = tmp.length / tmp.stride;
+    /* use max declsize */
+    if(dst->declsize > src->declsize) {
+        tmp.declsize  = dst->declsize;
+    } else {
+        tmp.declsize  = src->declsize;
+    }
     if(tmp.length % tmp.stride != 0) tmp.count++;
     if(tmp.first >= dst->stop || tmp.stop > dst->stop)
 	err = NC_EINVALCOORDS;
@@ -158,7 +156,7 @@ src into dst taking
 overlapping projections into acct.
 */
 int
-dapmergeprojections(NClist* dst, NClist* src)
+dcemergeprojectionlists(NClist* dst, NClist* src)
 {
     int i;
     NClist* cat = nclistnew();
@@ -197,7 +195,7 @@ fprintf(stderr,"dapmergeprojection: src = %s\n",dcetostring((DCEnode*)src));
 	    if(dcesamepath(target->var->segments,
 			   p2->var->segments)!=0) continue;
 	    /* This entry matches our current target; merge  */
-	    ncstat = mergeprojection(target,p2);
+	    ncstat = dcemergeprojections(target,p2);
 	    /* null out this merged entry and release it */
 	    nclistset(cat,i,(ncelem)NULL);	    
 	    dcefree((DCEnode*)p2);	    
@@ -209,30 +207,32 @@ fprintf(stderr,"dapmergeprojection: src = %s\n",dcetostring((DCEnode*)src));
     return ncstat;
 }
 
-static int
-mergeprojection(DCEprojection* dst, DCEprojection* src)
+/* Modify merged projection to include "addition" projection */
+int
+dcemergeprojections(DCEprojection* merged, DCEprojection* addition)
 {
     int ncstat = NC_NOERR;
     int i,j;
 
-    /* merge segment by segment;
-       |dst->segments| == |src->segments|
-       by construction
-    */
-    assert((dst->discrim == CES_VAR && src->discrim == CES_VAR));
-    assert((nclistlength(dst->var->segments) == nclistlength(src->var->segments)));    
-    for(i=0;i<nclistlength(dst->var->segments);i++) {
-	DCEsegment* dstseg = (DCEsegment*)nclistget(dst->var->segments,i);
-	DCEsegment* srcseg = (DCEsegment*)nclistget(src->var->segments,i);
-	for(j=0;j<dstseg->rank;j++) {
-	    dceslicemerge(dstseg->slices+j,
-			  srcseg->slices+j);
+    ASSERT((merged->discrim == CES_VAR && addition->discrim == CES_VAR));
+    ASSERT((nclistlength(merged->var->segments) == nclistlength(addition->var->segments)));    
+    for(i=0;i<nclistlength(merged->var->segments);i++) {
+	DCEsegment* mergedseg = (DCEsegment*)nclistget(merged->var->segments,i);
+	DCEsegment* addedseg = (DCEsegment*)nclistget(addition->var->segments,i);
+	/* If one segment has larger rank, then copy the extra slices unchanged */
+	for(j=0;j<addedseg->rank;j++) {
+	    if(j < mergedseg->rank)
+	        dceslicemerge(mergedseg->slices+j,addedseg->slices+j);
+	    else
+		mergedseg->slices[j] = addedseg->slices[j];
 	}
+	if(addedseg->rank > mergedseg->rank)
+	    mergedseg->rank = addedseg->rank;
     }
     return ncstat;
 }
 
-/* Convert an DCEprojection instance into a string
+/* Convert a DCEprojection instance into a string
    that can be used with the url
 */
 
@@ -479,6 +479,25 @@ dcetostring(DCEnode* node)
     return s;
 }
 
+/* For debugging */
+#ifdef DEBUG
+static char*
+dimdecl(size_t declsize)
+{
+    static char tag[16];
+    tag[0] = '\0';
+    if(dceverbose) 
+        snprintf(tag,sizeof(tag),"/%lu",(unsigned long)declsize);
+    return tag;
+}
+#else
+static char*
+dimdecl(size_t declsize)
+{
+    return "";
+}
+#endif
+
 void
 dcetobuffer(DCEnode* node, NCbytes* buf)
 {
@@ -486,27 +505,27 @@ dcetobuffer(DCEnode* node, NCbytes* buf)
     char tmp[1024];
 
     if(buf == NULL) return;
-    if(node == NULL) {ncbytesappend(buf,'?'); return;}
+    if(node == NULL) {ncbytescat(buf,"<null>"); return;}
 
     switch (node->sort) {
 
     case CES_SLICE: {
 	    DCEslice* slice = (DCEslice*)node;
 	    size_t last = (slice->first+slice->length)-1;
-	    if(last > slice->declsize && slice->declsize > 0)
-	        last = slice->declsize - 1;
             if(slice->count == 1) {
-                snprintf(tmp,sizeof(tmp),"[%lu]",
-	            (unsigned long)slice->first);
+                snprintf(tmp,sizeof(tmp),"[%lu%s]",
+	            (unsigned long)slice->first,dimdecl(slice->declsize));
             } else if(slice->stride == 1) {
-                snprintf(tmp,sizeof(tmp),"[%lu:%lu]",
+                snprintf(tmp,sizeof(tmp),"[%lu:%lu%s]",
 	            (unsigned long)slice->first,
-	            (unsigned long)last);
+	            (unsigned long)last,
+	            dimdecl(slice->declsize));
             } else {
-	        snprintf(tmp,sizeof(tmp),"[%lu:%lu:%lu]",
+	        snprintf(tmp,sizeof(tmp),"[%lu:%lu:%lu%s]",
 		    (unsigned long)slice->first,
 		    (unsigned long)slice->stride,
-		    (unsigned long)last);
+		    (unsigned long)last,
+	            dimdecl(slice->declsize));
 	    }
             ncbytescat(buf,tmp);
     } break;
@@ -515,9 +534,13 @@ dcetobuffer(DCEnode* node, NCbytes* buf)
 	DCEsegment* segment = (DCEsegment*)node;
         int rank = segment->rank;
 	char* name = (segment->name?segment->name:"<unknown>");
-	ncbytescat(buf,nulldup(name));
-        if(!dceiswholesegment(segment)) {
-	    for(i=0;i<rank;i++) {
+	name = nulldup(name);
+	ncbytescat(buf,name);
+	nullfree(name);
+        if(dceverbose && dceiswholesegment(segment))
+	    ncbytescat(buf,"*");
+        if(dceverbose || !dceiswholesegment(segment)) {
+            for(i=0;i<rank;i++) {
 	        DCEslice* slice = segment->slices+i;
                 dcetobuffer((DCEnode*)slice,buf);
 	    }
@@ -633,6 +656,7 @@ dcelisttobuffer(NClist* list, NCbytes* buf, char* sep)
     if(sep == NULL) sep = ",";
     for(i=0;i<nclistlength(list);i++) {
 	DCEnode* node = (DCEnode*)nclistget(list,i);
+	if(node == NULL) continue;
 	if(i>0) ncbytescat(buf,sep);
 	dcetobuffer((DCEnode*)node,buf);
     }
@@ -782,21 +806,12 @@ dcecreate(CEsort sort)
     return node;
 }
 
-void
-dcemakewholeslice(DCEslice* slice, size_t declsize)
-{
-    slice->first = 0;
-    slice->stride = 1;
-    slice->length = declsize;
-    slice->stop = declsize;
-    slice->declsize = declsize;
-    slice->count = declsize;
-}
-
 int
 dceiswholeslice(DCEslice* slice)
 {
-    if(slice->first != 0 || slice->stride != 1) return 0;
+    if(slice->first != 0
+       || slice->stride != 1
+       || slice->stop != slice->declsize) return 0;
     return 1;
 }
 
@@ -813,16 +828,27 @@ dceiswholesegment(DCEsegment* seg)
     return whole;
 }
 
-int
-dceiswholevar(DCEvar* var)
+void
+dcemakewholeslice(DCEslice* slice, size_t declsize)
 {
-    int i,whole;
-    whole = 1; /* assume so */
-    for(i=0;i<nclistlength(var->segments);i++) {
-        DCEsegment* segment = (DCEsegment*)nclistget(var->segments,i);
-	if(!dceiswholesegment(segment)) {whole = 0; break;}	
-    }
-    return whole;
+    slice->first = 0;
+    slice->stride = 1;
+    slice->length = declsize;
+    slice->stop = declsize;
+    slice->declsize = declsize;
+    slice->count = declsize;
+}
+
+/* Remove slicing from terminal segment of p */
+void
+dcemakewholeprojection(DCEprojection* p)
+{
+    /* Remove the slicing (if any) from the last segment */
+    if(p->discrim == CES_VAR && p->var != NULL && p->var->segments != NULL) {
+        int lastindex = nclistlength(p->var->segments) - 1;
+        DCEsegment* lastseg = (DCEsegment*)nclistget(p->var->segments,lastindex);
+        lastseg->rank = 0;
+    }   
 }
 
 int
