@@ -252,16 +252,21 @@ fprintf(stderr, "    REC %d %s: %ld\n", ii, (*vpp)->name->cp, (long)index);
 		    return NC_EVARSIZE;
 		}
 #endif
-		ncp->recsize += (*vpp)->len;
+		if((*vpp)->len != UINT32_MAX) /* flag for vars >= 2**32 bytes */
+		    ncp->recsize += (*vpp)->len;
 		last = (*vpp);
 	}
 
 	/*
-	 * for special case of exactly one record variable, pack value
+	 * for special case of 
 	 */
-	if(last != NULL && ncp->recsize == last->len)
+	if(last != NULL) {
+	    if(ncp->recsize == last->len) { /* exactly one record variable, pack value */
 		ncp->recsize = *last->dsizes * last->xsz;
-
+	    } else if(last->len == UINT32_MAX) { /* huge last record variable */
+		ncp->recsize += *last->dsizes * last->xsz;
+	    }
+	}
 	if(NC_IsNew(ncp))
 		NC_set_numrecs(ncp, 0);
 	return NC_NOERR;
@@ -283,7 +288,7 @@ read_numrecs(NC *ncp)
 
 #define NC_NUMRECS_OFFSET 4
 #define NC_NUMRECS_EXTENT 4
-	status = ncp->nciop->get(ncp->nciop,
+	status = ncio_get(ncp->nciop,
 		 NC_NUMRECS_OFFSET, NC_NUMRECS_EXTENT, 0, (void **)&xp);
 					/* cast away const */
 	if(status != NC_NOERR)
@@ -291,7 +296,7 @@ read_numrecs(NC *ncp)
 
 	status = ncx_get_size_t(&xp, &nrecs);
 
-	(void) ncp->nciop->rel(ncp->nciop, NC_NUMRECS_OFFSET, 0);
+	(void) ncio_rel(ncp->nciop, NC_NUMRECS_OFFSET, 0);
 
 	if(status == NC_NOERR)
 	{
@@ -316,7 +321,7 @@ write_numrecs(NC *ncp)
 	assert(!NC_readonly(ncp));
 	assert(!NC_indef(ncp));
 
-	status = ncp->nciop->get(ncp->nciop,
+	status = ncio_get(ncp->nciop,
 		 NC_NUMRECS_OFFSET, NC_NUMRECS_EXTENT, RGN_WRITE, &xp);
 	if(status != NC_NOERR)
 		return status;
@@ -326,7 +331,7 @@ write_numrecs(NC *ncp)
 		status = ncx_put_size_t(&xp, &nrecs);
 	}
 
-	(void) ncp->nciop->rel(ncp->nciop, NC_NUMRECS_OFFSET, RGN_MODIFIED);
+	(void) ncio_rel(ncp->nciop, NC_NUMRECS_OFFSET, RGN_MODIFIED);
 
 	if(status == NC_NOERR)
 		fClr(ncp->flags, NC_NDIRTY);
@@ -546,7 +551,7 @@ move_recs_r(NC *gnu, NC *old)
 
 		assert(gnu_off > old_off);
 	
-		status = gnu->nciop->move(gnu->nciop, gnu_off, old_off,
+		status = ncio_move(gnu->nciop, gnu_off, old_off,
 			 old_varp->len, 0);
 
 		if(status != NC_NOERR)
@@ -598,7 +603,7 @@ move_vars_r(NC *gnu, NC *old)
 
 		assert(gnu_off > old_off);
 
-		status = gnu->nciop->move(gnu->nciop, gnu_off, old_off,
+		status = ncio_move(gnu->nciop, gnu_off, old_off,
 			 old_varp->len, 0);
 
 		if(status != NC_NOERR)
@@ -783,7 +788,7 @@ NC_endef(NC *ncp,
 
 	fClr(ncp->flags, NC_CREAT | NC_INDEF);
 
-	return ncp->nciop->sync(ncp->nciop);
+	return ncio_sync(ncp->nciop);
 }
 
 #ifdef LOCKNUMREC
@@ -918,10 +923,9 @@ NC3_create(const char *path, int ioflags,
 
 	assert(ncp->xsz == ncx_len_NC(ncp,sizeof_off_t));
 	
-	status = ncio_create(path, ioflags,
-		initialsz,
-		0, ncp->xsz, &ncp->chunk,
-		&ncp->nciop, &xp);
+        status =  ncio_create(path, ioflags, initialsz,
+			      0, ncp->xsz, &ncp->chunk,
+			      &ncp->nciop, &xp);
 	if(status != NC_NOERR)
 	{
 		/* translate error status */
@@ -1022,9 +1026,7 @@ NC3_open(const char * path, int ioflags,
 		return NC_EINVAL;
 #endif
 
-	status = ncio_open(path, ioflags,
-		0, 0, &ncp->chunk,
-		&ncp->nciop, 0);
+	status = ncio_open(path, ioflags, 0, 0, &ncp->chunk, &ncp->nciop, 0);
 	if(status)
 		goto unwind_alloc;
 
@@ -1051,7 +1053,6 @@ NC3_open(const char * path, int ioflags,
 	if(chunksizehintp != NULL)
 		*chunksizehintp = ncp->chunk;
 
-
 	ncp->int_ncid = ncp->nciop->fd;
 
 	if(ncpp) *ncpp = ncp;
@@ -1109,7 +1110,7 @@ NC3_close(int ncid)
 	{
 		status = NC_sync(ncp);
 		/* flush buffers before any filesize comparisons */
-		(void) ncp->nciop->sync(ncp->nciop);
+		(void) ncio_sync(ncp->nciop);
 	}
 
 	/* 
@@ -1289,7 +1290,7 @@ NC3_sync(int ncid)
 	if(status != NC_NOERR)
 		return status;
 
-	status = ncp->nciop->sync(ncp->nciop);
+	status = ncio_sync(ncp->nciop);
 	if(status != NC_NOERR)
 		return status;
 
@@ -1365,19 +1366,21 @@ NC_get_numrecs(const NC *ncp) {
 }
 
 void
-NC_set_numrecs(NC *ncp, size_t nrecs) {
-	shmem_t numrec = (shmem_t) nrecs;
-	/* update local value too */
-	ncp->lock[LOCKNUMREC_VALUE] = (ushmem_t) numrec;
-	shmem_short_put((shmem_t *) ncp->lock + LOCKNUMREC_VALUE, &numrec, 1,
-		ncp->lock[LOCKNUMREC_BASEPE]);
+NC_set_numrecs(NC *ncp, size_t nrecs)
+{
+    shmem_t numrec = (shmem_t) nrecs;
+    /* update local value too */
+    ncp->lock[LOCKNUMREC_VALUE] = (ushmem_t) numrec;
+    shmem_short_put((shmem_t *) ncp->lock + LOCKNUMREC_VALUE, &numrec, 1,
+    ncp->lock[LOCKNUMREC_BASEPE]);
 }
 
-void NC_increase_numrecs(NC *ncp, size_t nrecs) {
-	/* this is only called in one place that's already protected
-	 * by a lock ... so don't worry about it */
-	if (nrecs > NC_get_numrecs(ncp))
-		NC_set_numrecs(ncp, nrecs);
+void NC_increase_numrecs(NC *ncp, size_t nrecs)
+{
+    /* this is only called in one place that's already protected
+     * by a lock ... so don't worry about it */
+    if (nrecs > NC_get_numrecs(ncp))
+	NC_set_numrecs(ncp, nrecs);
 }
 
 #endif /* LOCKNUMREC */
@@ -1519,6 +1522,7 @@ nc_delete_mp(const char * path, int basepe)
 	if(basepe != 0)
 		return NC_EINVAL;
 #endif
+
 	status = ncio_open(path, NC_NOWRITE,
 		0, 0, &ncp->chunk,
 		&ncp->nciop, 0);
@@ -1539,6 +1543,7 @@ nc_delete_mp(const char * path, int basepe)
 		/* ncio_close does the unlink */
 		status = ncio_close(ncp->nciop, 1); /* ncio_close does the unlink */
 	}
+	ncp->nciop = NULL;
 
 	ncp->nciop = NULL;
 unwind_alloc:
@@ -1549,7 +1554,6 @@ unwind_alloc:
 int
 nc_delete(const char * path)
 {
-	return nc_delete_mp(path, 0);
+        return nc_delete_mp(path, 0);
 }
 
-
