@@ -2,28 +2,39 @@
    See the COPYRIGHT file for more information. */
 
 #include "config.h"
-#include "dapparselex.h"
 #include <strings.h>
+#include "dapparselex.h"
 
-#define URLCVT
-#define NONSTDCVT
+#undef URLCVT /* NEVER turn this on */
+#define DAP2ENCODE
 
 /* Forward */
 static void dumptoken(DAPlexstate* lexstate);
-static int tohex(int c);
 static void dapaddyytext(DAPlexstate* lex, int c);
+#ifndef DAP2ENCODE
+static int tohex(int c);
+#endif
 
 /****************************************************/
+
+#if 0 /* Following definitions are for informational purposes */
+/* Set of all ascii printable characters */
+static char ascii[] = " !\"#$%&'()*+,-./:;<=>?@[]\\^_`|{}~";
+
+/* Define the set of legal nonalphanum characters as specified in the DAP2 spec. */
+static char* daplegal ="_!~*'-\"";
+#endif
+
 static char* ddsworddelims =
   "{}[]:;=,";
 
 /* Define 1 and > 1st legal characters */
 static char* ddswordchars1 =
-  "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-+_/%.\\*";
+  "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-+_/%\\.*";
 static char* ddswordcharsn =
-  "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-+_/%.\\*#";
+  "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-+_/%\\.*#";
 static char* daswordcharsn =
-  "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-+_/%.\\*:#";
+  "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-+_/%\\.*#:";
 static char* cewordchars1 =
   "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-+_/%\\";
 static char* cewordcharsn =
@@ -36,9 +47,6 @@ static char* wordcharsn = NULL;
 static char* worddelims = NULL;
 */
 
-/* Hex digits */
-static char hexdigits[] = "0123456789abcdefABCDEF";
-
 static char* keywords[] = {
 "alias",
 "array",
@@ -99,7 +107,7 @@ daplex(YYSTYPE* lvalp, DAPparsestate* state)
     int token;
     int c;
     unsigned int i;
-    char* p=lexstate->next;
+    char* p;
     char* tmp;
 
     token = 0;
@@ -118,9 +126,18 @@ daplex(YYSTYPE* lvalp, DAPparsestate* state)
 	    token = c;
 	} else if(c == '"') {
 	    int more = 1;
-	    /* We have a string token; will be reported as SCAN_WORD */
+	    /* We have a string token; will be reported as WORD_STRING */
 	    while(more && (c=*(++p))) {
-#ifdef NONSTDCVT
+#ifdef DAP2ENCODE
+	        if(c == '"')
+		    more = 0;
+		else if(c == '\\') {
+		    /* Remove spec ambiguity by convering \c to c
+                       for any character c */
+		    c=*(++p);
+		    if(c == '\0') more = 0;
+		}
+#else /*Non-standard*/
 		switch (c) {
 		case '"': more=0; break;
 		case '\\':
@@ -151,27 +168,19 @@ daplex(YYSTYPE* lvalp, DAPparsestate* state)
 		    break;
 		default: break;
 		}
-#else /*!NONSTDCVT*/
-	        if(c == '"')
-		    more = 0;
-		else if(c == '\\') {
-		    c=*(++p);
-		    if(c == '\0') more = false;
-		    if(c != '"') {c = '\\'; --p;}
-		}
-#endif /*!NONSTDCVT*/
+#endif /*!DAP2ENCODE*/
 		if(more) dapaddyytext(lexstate,c);
 	    }
-	    token=SCAN_WORD;
+	    token=WORD_STRING;
 	} else if(strchr(lexstate->wordchars1,c) != NULL) {
-	    /* we have a SCAN_WORD */
+	    int isdatamark = 0;
+	    /* we have a WORD_WORD */
 	    dapaddyytext(lexstate,c);
 	    while((c=*(++p))) {
 #ifdef URLCVT
 		if(c == '%' && p[1] != 0 && p[2] != 0
 			    && strchr(hexdigits,p[1]) != NULL
                             && strchr(hexdigits,p[2]) != NULL) {
-#ifdef WRONG /* Should not unescape %xx occurrences */
 		    int d1,d2;
 		    d1 = tohex(p[1]);
 		    d2 = tohex(p[2]);
@@ -179,7 +188,6 @@ daplex(YYSTYPE* lvalp, DAPparsestate* state)
 			c=(((unsigned int)d1)<<4) | (unsigned int)d2;
 			p+=2;
 		    }
-#endif
 		} else {
 		    if(strchr(lexstate->wordcharsn,c) == NULL) {p--; break;}
 		}
@@ -193,10 +201,19 @@ daplex(YYSTYPE* lvalp, DAPparsestate* state)
 	    tmp = ocbytescontents(lexstate->yytext);
 	    if(strcmp(tmp,"Data")==0 && *p == ':') {
 		dapaddyytext(lexstate,*p); p++;
-		token = SCAN_DATA;
-	    } else {
+		if(p[0] == '\n') {
+		    token = SCAN_DATA;
+		    isdatamark = 1;
+		    p++;
+	        } else if(p[0] == '\r' && p[1] == '\n') {
+		    token = SCAN_DATA;
+		    isdatamark = 1;
+		    p+=2;
+		}
+	    }
+	    if(!isdatamark) {
 	        /* check for keyword */
-	        token=SCAN_WORD; /* assume */
+	        token=WORD_WORD; /* assume */
 	        for(i=0;;i++) {
 		    if(keywords[i] == NULL) break;
 		    if(strcasecmp(keywords[i],tmp)==0) {
@@ -231,6 +248,7 @@ dapaddyytext(DAPlexstate* lex, int c)
     ocbytesappend(lex->yytext,(char)c);
 }
 
+#ifndef DAP2ENCODE
 static int
 tohex(int c)
 {
@@ -239,6 +257,7 @@ tohex(int c)
     if(c >= '0' && c <= '9') return (c - '0');
     return -1;
 }
+#endif
 
 static void
 dumptoken(DAPlexstate* lexstate)
@@ -304,3 +323,28 @@ daplexcleanup(DAPlexstate** lexstatep)
     free(lexstate);
     *lexstatep = NULL;
 }
+
+/* Dap identifiers will come to us with some
+   characters escaped using the URL notation of
+   %HH. The assumption here is that any character
+   that is encoded is left encoded, except as follows:
+   1. if the encoded character is in fact a legal DAP2 character
+      (alphanum+"_!~*'-\"") then it is decoded, otherwise not.
+*/
+#ifndef DECODE_IDENTIFIERS
+static char* decodelist =
+  "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_!~*'-\"";
+#endif
+
+char*
+dapdecode(DAPlexstate* lexstate, char* name)
+{
+    char* decoded;
+#ifdef DECODE_IDENTIFIERS
+    decoded = ocuridecode(name);
+#else
+    decoded = ocuridecodeonly(name,decodelist);
+#endif
+    oclistpush(lexstate->reclaim,(ocelem)decoded);
+    return decoded;
+}
